@@ -196,25 +196,28 @@ def test_multipublish():
         conn.multipublish('topic', ['sup', 'sup'])
 
 
-@pytest.mark.parametrize('error,error_class,fatal', [
-    ('E_INVALID', errors.NSQInvalid, True),
-    ('E_BAD_BODY', errors.NSQBadBody, True),
-    ('E_BAD_TOPIC', errors.NSQBadTopic, True),
-    ('E_BAD_CHANNEL', errors.NSQBadChannel, True),
-    ('E_BAD_MESSAGE', errors.NSQBadMessage, True),
-    ('E_PUT_FAILED', errors.NSQPutFailed, True),
-    ('E_PUB_FAILED', errors.NSQPubFailed, True),
-    ('E_MPUB_FAILED', errors.NSQMPubFailed, True),
-    ('E_FIN_FAILED', errors.NSQFinishFailed, False),
-    ('E_REQ_FAILED', errors.NSQRequeueFailed, False),
-    ('E_TOUCH_FAILED', errors.NSQTouchFailed, False),
-    ('unknown error', errors.NSQException, True),
+@pytest.mark.parametrize('error_msg,error,fatal', [
+    ('E_INVALID cannot SUB in current state', 'NSQInvalid', True),
+    ('E_BAD_BODY MPUB failed to read body size', 'NSQBadBody', True),
+    ('E_BAD_TOPIC SUB topic name oh my god is not valid', 'NSQBadTopic', True),
+    ('E_BAD_CHANNEL SUB channel name !! is not valid', 'NSQBadChannel', True),
+    ('E_BAD_MESSAGE PUB failed to read message body', 'NSQBadMessage', True),
+    ('E_PUT_FAILED PUT failed', 'NSQPutFailed', True),
+    ('E_PUB_FAILED PUB failed', 'NSQPubFailed', True),
+    ('E_MPUB_FAILED MPUB failed', 'NSQMPubFailed', True),
+    ('E_AUTH_DISABLED AUTH Disabled', 'NSQAuthDisabled', True),
+    ('E_AUTH_FAILED AUTH failed', 'NSQAuthFailed', True),
+    ('E_UNAUTHORIZED AUTH No authorizations found', 'NSQUnauthorized', True),
+    ('E_FIN_FAILED FIN failed', 'NSQFinishFailed', False),
+    ('E_REQ_FAILED REQ failed', 'NSQRequeueFailed', False),
+    ('E_TOUCH_FAILED TOUCH failed', 'NSQTouchFailed', False),
+    ('some unknown error', 'NSQException', True),
 ])
-def test_error(error, error_class, fatal):
+def test_error(error_msg, error, fatal):
     @mock_server
     def handle(socket, address):
         assert socket.recv(4) == '  V2'
-        socket.send(mock_response(nsq.FRAME_TYPE_ERROR, error))
+        socket.send(mock_response(nsq.FRAME_TYPE_ERROR, error_msg))
 
     with handle as server:
         conn = Nsqd(address='127.0.0.1', tcp_port=server.server_port)
@@ -222,7 +225,7 @@ def test_error(error, error_class, fatal):
 
         frame, resp = conn.read_response()
         assert frame == nsq.FRAME_TYPE_ERROR
-        assert isinstance(resp, error_class)
+        assert isinstance(resp, getattr(errors, error))
         assert conn.is_connected != fatal
 
 
@@ -300,6 +303,71 @@ def test_sync_heartbeat():
         assert data == '_heartbeat_'
 
 
+def test_auth():
+    @mock_server
+    def handle(socket, address):
+        assert socket.recv(4) == '  V2'
+        assert socket.recv(5) == 'AUTH\n'
+
+        assert nsq.unpack_size(socket.recv(4)) == 6
+        assert socket.recv(6) == 'secret'
+
+        resp = json.dumps({'identity': 'awesome'})
+        socket.send(mock_response(nsq.FRAME_TYPE_RESPONSE, resp))
+
+    with handle as server:
+        conn = Nsqd(
+            address='127.0.0.1',
+            tcp_port=server.server_port,
+            auth_secret='secret'
+        )
+
+        conn.connect()
+        resp = conn.auth()
+        assert resp['identity'] == 'awesome'
+
+
+def test_identify_auth():
+    @mock_server
+    def handle(socket, address):
+        assert socket.recv(4) == '  V2'
+        assert socket.recv(9) == 'IDENTIFY\n'
+
+        size = nsq.unpack_size(socket.recv(4))
+        data = json.loads(socket.recv(size))
+        assert 'gnsq' in data['user_agent']
+
+        resp = json.dumps({'auth_required': True})
+        socket.send(mock_response(nsq.FRAME_TYPE_RESPONSE, resp))
+
+        assert socket.recv(5) == 'AUTH\n'
+        assert nsq.unpack_size(socket.recv(4)) == 6
+        assert socket.recv(6) == 'secret'
+
+        resp = json.dumps({'identity': 'awesome'})
+        socket.send(mock_response(nsq.FRAME_TYPE_RESPONSE, resp))
+
+    with handle as server:
+        conn = Nsqd(
+            address='127.0.0.1',
+            tcp_port=server.server_port,
+            auth_secret='secret'
+        )
+
+        @conn.on_auth.connect
+        def assert_auth(conn, response):
+            assert assert_auth.was_called is False
+            assert_auth.was_called = True
+            assert response['identity'] == 'awesome'
+
+        assert_auth.was_called = False
+        conn.connect()
+        resp = conn.identify()
+
+        assert resp['auth_required']
+        assert assert_auth.was_called
+
+
 @pytest.mark.parametrize('tls,deflate,snappy', product((True, False), repeat=3))
 @pytest.mark.slow
 def test_socket_upgrades(tls, deflate, snappy):
